@@ -1,27 +1,37 @@
+import argparse
 import sys
 from collections import defaultdict
 from dataclasses import dataclass
-from typing import cast, Dict, Iterator, List, Optional
+from typing import cast, Dict, Iterator, List, Optional, Type, TypeVar
 
 import cbor2
+import geometry
+import geometry as g
 import numpy as np
 import yaml
 from cbor2 import CBORDecodeEOF
+from zuper_commons.types import ZException
+from zuper_ipce import IEDO, object_from_ipce
+from zuper_ipce.json2cbor import tag_hook
 
-import geometry
-from aido_schemas.protocol_simulator import RobotObservations, RobotState, SetRobotCommands
-from aido_schemas.schemas import DTSimRobotInfo, Duckiebot1Observations
+import duckietown_world as dw
+from aido_schemas.protocol_simulator import (
+    RobotName,
+    RobotObservations,
+    RobotState,
+    SetRobotCommands,
+    SpawnDuckie,
+    SpawnRobot,
+)
+from aido_schemas.schemas import DB20Observations, DTSimRobotInfo
 from duckietown_world import draw_static, DuckietownMap, SampledSequence, SE2Transform
 from duckietown_world.rules import evaluate_rules
 from duckietown_world.rules.rule import make_timeseries, RuleEvaluationResult
 from duckietown_world.seqs.tsequence import SampledSequenceBuilder
 from duckietown_world.svg_drawing.draw_log import RobotTrajectories, SimulatorLog
 from duckietown_world.svg_drawing.misc import TimeseriesPlot
-from duckietown_world.world_duckietown import construct_map, DB18
-from duckietown_world.world_duckietown.types import SE2v
+from duckietown_world.world_duckietown import construct_map, DB18, Duckie
 from duckietown_world.world_duckietown.utils import get_velocities_from_sequence
-from zuper_ipce import IEDO, object_from_ipce
-from zuper_ipce.json2cbor import tag_hook
 from . import logger
 
 
@@ -30,8 +40,9 @@ class LogData:
     objects: List[dict]
 
 
-def log_summary(filename: str) -> LogData:
+def log_summary(filename: str, measure_size: bool = True) -> LogData:
     objects = []
+
     with open(filename, "rb") as f:
         counts = defaultdict(lambda: 0)
         sizes = defaultdict(lambda: 0)
@@ -44,8 +55,10 @@ def log_summary(filename: str) -> LogData:
 
             # for ob in read_cbor_or_json_objects(f):
             topic = ob["topic"]
-            size_ob = 0
-            # size_ob = len(cbor2.dumps(ob))
+            if measure_size:
+                size_ob = len(cbor2.dumps(ob))
+            else:
+                size_ob = 0
             counts[topic] += 1
             sizes[topic] += size_ob
             objects.append(ob)
@@ -54,7 +67,7 @@ def log_summary(filename: str) -> LogData:
     for topic in ordered:
         count = counts[topic]
         size_mb = sizes[topic] / (1024 * 1024.0)
-        logger.info("topic %25s: %4s messages  %.2f MB" % (topic, count, size_mb))
+        logger.info(f"topic {topic:>25}: {count:>4} messages  {size_mb:.2f} MB")
     return LogData(objects)
 
 
@@ -64,6 +77,17 @@ def read_topic2(ld: LogData, topic: str) -> Iterator[dict]:
             yield ob
 
 
+X = TypeVar("X")
+
+
+def read_topic_as(ld: LogData, topic: str, K: Type[X]) -> Iterator[X]:
+    for ob in ld.objects:
+        if ob["topic"] == topic:
+            data = ob["data"]
+            inter = object_from_ipce(data, K, iedo=iedo)
+            yield inter
+
+
 def read_map_info(ld: LogData) -> DuckietownMap:
     m = list(read_topic2(ld, "set_map"))
     if not m:
@@ -79,9 +103,7 @@ def read_map_info(ld: LogData) -> DuckietownMap:
 def read_perfomance(ld: LogData) -> Dict[str, RuleEvaluationResult]:
     sb = SampledSequenceBuilder[float]()
     sb.add(0, 0)
-    sequences: Dict[str, SampledSequenceBuilder] = defaultdict(
-        lambda: SampledSequenceBuilder[float]()
-    )
+    sequences: Dict[str, SampledSequenceBuilder] = defaultdict(lambda: SampledSequenceBuilder[float]())
 
     for i, ob in enumerate(read_topic2(ld, "timing_information")):
         # ob = ipce_to_object(ob['data'], {}, {})
@@ -90,6 +112,7 @@ def read_perfomance(ld: LogData) -> Dict[str, RuleEvaluationResult]:
         for p, f in phases.items():
             sequences[p].add(t=i, v=f)
 
+    # noinspection PyTypeChecker XXX
     evr = RuleEvaluationResult(None)
     for p, sb in sequences.items():
         seq = sb.as_sequence()
@@ -99,18 +122,18 @@ def read_perfomance(ld: LogData) -> Dict[str, RuleEvaluationResult]:
     return {"performance": evr}
 
 
-def read_trajectories(ld: LogData) -> Dict[str, RobotTrajectories]:
+def read_trajectories(ld: LogData) -> Dict[RobotName, RobotTrajectories]:
     rs = list(read_topic2(ld, "robot_state"))
     if not rs:
         msg = "Could not find robot_state"
         raise Exception(msg)
     robot_names = set([r["data"]["robot_name"] for r in rs])
-    logger.info(f"robot_names: {robot_names}")
+    logger.info(robot_names=robot_names)
 
     robot2trajs = {}
     for robot_name in robot_names:
         ssb_pose = SampledSequenceBuilder[SE2Transform]()
-        ssb_pose_SE2 = SampledSequenceBuilder[SE2v]()
+        ssb_pose_SE2 = SampledSequenceBuilder[g.SE2value]()
         # ssb_actions = SampledSequenceBuilder[object]()
         # ssb_wheels_velocities = SampledSequenceBuilder[object]()
         # ssb_velocities = SampledSequenceBuilder[Any]()
@@ -157,7 +180,7 @@ def read_observations(ld: LogData, robot_name: str) -> SampledSequence:
         ro = cast(RobotObservations, found)
         if ro.robot_name != robot_name:
             continue
-        do = cast(Duckiebot1Observations, ro.observations)
+        do = cast(DB20Observations, ro.observations)
 
         t = ro.t_effective
         camera = do.camera.jpg_data
@@ -192,35 +215,36 @@ def read_commands(ld: LogData, robot_name: str) -> SampledSequence:
     return seq
 
 
+def read_simulator_log_cbor(ld: LogData, main_robot_name: Optional[str] = None) -> SimulatorLog:
+    robot_spawn: Dict[str, SpawnRobot] = {
+        v.robot_name: v for v in read_topic_as(ld, "spawn_robot", SpawnRobot)
+    }
+    duckie_spawn: Dict[str, SpawnDuckie] = {x.name: x for x in read_topic_as(ld, "spawn_duckie", SpawnDuckie)}
+    logger.info(spawn=robot_spawn, duckie_spawn=duckie_spawn)
 
-def read_simulator_log_cbor(
-    ld: LogData, main_robot_name: Optional[str] = None
-) -> SimulatorLog:
     render_time = read_perfomance(ld)
     duckietown_map = read_map_info(ld)
     robots = read_trajectories(ld)
     # logger.info(f'robots: {len(robots)}')
 
+    for duckie_name, ds in duckie_spawn.items():
+        color = ds.color
+        pose = ds.pose
+        duckie = Duckie(color=color)
+
+        pose2 = SE2Transform.from_SE2(pose)
+
+        gt = dw.Constant[dw.SE2Transform](pose2)
+        duckietown_map.set_object(duckie_name, duckie, ground_truth=gt)
+
     for robot_name, trajs in robots.items():
-        # logger.info(f'robots: {robot_name} trajs: {trajs.pose.get_sampling_points()}')
-        if robot_name == main_robot_name:
-            color = "red"
-        elif "ego" in robot_name:
-            color = "pink"
-        elif "parked" in robot_name:
-            color = "blue"
-        elif "npc" in robot_name:
-            color = "yellow"
-        else:
-            color = "grey"
+        color = robot_spawn[robot_name].color
 
         robot = DB18(color=color)
         # noinspection PyTypeChecker
         duckietown_map.set_object(robot_name, robot, ground_truth=trajs.pose)
 
-    return SimulatorLog(
-        duckietown=duckietown_map, robots=robots, render_time=render_time
-    )
+    return SimulatorLog(duckietown=duckietown_map, robots=robots, render_time=render_time)
 
 
 def evaluate_stats(fn: str, robot_main: str) -> Dict[str, RuleEvaluationResult]:
@@ -230,17 +254,12 @@ def evaluate_stats(fn: str, robot_main: str) -> Dict[str, RuleEvaluationResult]:
     duckietown_env = log0.duckietown
     interval = SampledSequence.from_iterator(enumerate(log.pose.timestamps))
     evaluated = evaluate_rules(
-        poses_sequence=log.pose,
-        interval=interval,
-        world=duckietown_env,
-        ego_name=robot_main,
+        poses_sequence=log.pose, interval=interval, world=duckietown_env, ego_name=robot_main,
     )
     return evaluated
 
 
-def read_and_draw(
-    fn: str, output: str, robot_main: str
-) -> Dict[str, RuleEvaluationResult]:
+def read_and_draw(fn: str, output: str, robot_main: str) -> Dict[str, RuleEvaluationResult]:
     ld = log_summary(fn)
 
     logger.info("Reading logs...")
@@ -250,7 +269,7 @@ def read_and_draw(
     # for robot_main in pc_names:
     if not robot_main in log0.robots:
         msg = f"Cannot find robot {robot_main!r}"
-        raise Exception(msg)
+        raise ZException(msg, known=sorted(log0.robots))
 
     log = log0.robots[robot_main]
 
@@ -270,10 +289,7 @@ def read_and_draw(
     logger.info("Evaluating rules...")
     interval = SampledSequence.from_iterator(enumerate(log.pose.timestamps))
     evaluated = evaluate_rules(
-        poses_sequence=log.pose,
-        interval=interval,
-        world=duckietown_env,
-        ego_name=robot_main,
+        poses_sequence=log.pose, interval=interval, world=duckietown_env, ego_name=robot_main,
     )
     if True:
         evaluated.update(log0.render_time)
@@ -284,19 +300,13 @@ def read_and_draw(
     timeseries.update(make_timeseries(evaluated))
     logger.info("Drawing...")
     draw_static(
-        duckietown_env,
-        output,
-        images=images,
-        timeseries=timeseries,
-        main_robot_name=robot_main,
+        duckietown_env, output, images=images, timeseries=timeseries, main_robot_name=robot_main,
     )
     logger.info("...done.")
     return evaluated
 
 
-def timeseries_wheels_velocities(
-    log_commands: SampledSequence,
-) -> Dict[str, TimeseriesPlot]:
+def timeseries_wheels_velocities(log_commands: SampledSequence,) -> Dict[str, TimeseriesPlot]:
     timeseries = {}
     sequences = {}
 
@@ -314,9 +324,7 @@ def timeseries_wheels_velocities(
     return timeseries
 
 
-def timeseries_robot_velocity(
-    log_velocity: SampledSequence,
-) -> Dict[str, TimeseriesPlot]:
+def timeseries_robot_velocity(log_velocity: SampledSequence,) -> Dict[str, TimeseriesPlot]:
     timeseries = {}
     sequences = {}
 
@@ -331,9 +339,7 @@ def timeseries_robot_velocity(
         return omega
 
     sequences["linear_speed"] = log_velocity.transform_values(lambda _: speed(_), float)
-    sequences["angular_velocity"] = log_velocity.transform_values(
-        lambda _: omega(_), float
-    )
+    sequences["angular_velocity"] = log_velocity.transform_values(lambda _: omega(_), float)
     # logger.info("linear speed: %s" % sequences["linear_speed"])
     # logger.info("angular velocity: %s" % sequences["angular_velocity"])
     timeseries["velocity"] = TimeseriesPlot("Velocities", "velocities", sequences)
@@ -347,3 +353,14 @@ def aido_log_draw_main():
 if __name__ == "__main__":
     # in case called
     read_and_draw(sys.argv[2], "test", robot_main=sys.argv[3])
+
+
+def aido_log_draw_main(args=None):
+    parser = argparse.ArgumentParser(description="")
+
+    parser.add_argument("--gslog", required=True)
+    parser.add_argument("--robot", required=True)
+    parser.add_argument("--outdir", help="output directory location", required=True)
+
+    args = parser.parse_args(args)
+    read_and_draw(args.gslog, args.outdir, robot_main=args.robot)
